package org.geekfu.MapPanel;

import info.clearthought.layout.TableLayout;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Image;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.io.IOException;
import java.util.LinkedList;
import java.util.List;

import javax.imageio.ImageIO;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;

import org.geekfu.OldCartographer.Map;

public class MapPanel extends JPanel implements ActionListener{

	protected Map map;
	protected JScrollPane mapscroll;
	protected JPanel sidebar;

	protected FireableTextField textentry;
	protected JTextArea textoutput;
	protected JScrollPane textscroll;
	
	protected JFrame textWindow;
	protected JTextArea windowTextEntry;
	
	protected JSplitPane split;
	
	protected List<MapPanelListener> listeners=new LinkedList<MapPanelListener>();

	private boolean text;

	public static final int BORDER=10;
	private static final String IMAGEPATH = "/org/geekfu/hippo/blueHippo.png";

	/**
	 * @param arg0
	 * @param arg1
	 * @param map
	 * @param sidebar
	 * @param text
	 */
	public MapPanel(Map map, JPanel sidebar, boolean text) {
		super();

		init(map,sidebar,text);
	}

	protected MapPanel() {}

	protected void init(Map map) {
		init(map,null,false);
	}

	protected void init(Map map, JPanel sidebar, boolean text) {
		this.map = map;
		this.sidebar = sidebar;
		this.text=text;

		textentry=new FireableTextField();
		textoutput=new JTextArea();
		textoutput.setEditable(false);
		textscroll = new JScrollPane(textoutput);
		textscroll.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		
		layoutWindow();
	}

	
	private void layoutWindow(){
		JComponent mapComponent=map;
		
		if(mapComponent==null){
			Image img;
			try {
				img = ImageIO.read(getClass().getResource(IMAGEPATH));
				mapComponent=new ImageComponent(img);
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		
		mapscroll=new JScrollPane(new CenteredPanel(mapComponent));
		if(map!=null){
			mapscroll.getVerticalScrollBar().setUnitIncrement(map.getCellsize().height/2);
			mapscroll.getHorizontalScrollBar().setUnitIncrement(map.getCellsize().width/2);
		}
		
		if(text){ ///////////////////////////////////// With text widgets
			split=new JSplitPane(JSplitPane.VERTICAL_SPLIT);
			split.add(mapscroll);
			split.add(textscroll);
			split.setDividerLocation(400);
			split.setResizeWeight(1.0);
			
			double[][] sizes= {
					{BORDER, TableLayout.FILL, TableLayout.PREFERRED, TableLayout.PREFERRED, BORDER},
					{
						BORDER, 
						TableLayout.FILL, 
						TableLayout.PREFERRED, 
						BORDER},
			};
			setLayout(new TableLayout(sizes));

			// Add split pane
			add(split,"1, 1, 2, 1");
			
			// Add text entry and button
			add(textentry,"1, 2, 1, 2");
			JButton showTextWindow=new JButton("...");
			showTextWindow.addActionListener(this);
			add(showTextWindow,"2, 2, 2, 2");
			
			// Add sidebar if exists
			if(sidebar!=null) {
				add(sidebar,"3, 1, 3, 2");
			}
		}else { ///////////////////////////////////// Without text widgets
			double[][] sizes= {
					{BORDER, TableLayout.FILL, TableLayout.PREFERRED, BORDER},
					{
						BORDER, 
						TableLayout.FILL, 
						BORDER},
			};
			setLayout(new TableLayout(sizes));

			add(mapscroll,"1, 1");
			if(sidebar!=null) {
				add(sidebar,"2, 1");
			}
		}
		
		validate();
	}
	
	
	public void scrollToCell(int x,int y) {
		if(map==null)return;
		Dimension c=map.getCellsize();
		Rectangle r=new Rectangle(x*c.width,y*c.height,c.width,c.height);
		map.scrollRectToVisible(r);
	}


	public void addActionListener(MapPanelListener a) {
		listeners.add(a);
	}

	public void showString(String message){
		if(!text)return;
		textoutput.append(message);
		if(!message.endsWith("\n"))
			textoutput.append("\n");
		textoutput.scrollRectToVisible(new Rectangle(0,textoutput.getHeight(),1,1));
	}
	
	public Map getMap() {
		return map;
	}

	public JPanel getSidebar() {
		return sidebar;
	}

	public boolean isText() {
		return text;
	}

	public void setMap(Map map) {
		this.map = map;
		layoutWindow();
	}

	public void setSidebar(JPanel sidebar) {
		this.sidebar = sidebar;
	}

	public void setText(boolean text) {
		this.text = text;
		((TableLayout) getLayout()).setRow(2,(isText()?TableLayout.PREFERRED:0));
		((TableLayout) getLayout()).setRow(3,(isText()?TableLayout.PREFERRED:0));
		layoutWindow();
	}

	public void actionPerformed(ActionEvent arg0) {
		if(textWindow==null){
			textWindow=new JFrame();
			textWindow.setSize(300,400);
			windowTextEntry=new JTextArea();
			
			JScrollPane textscroll = new JScrollPane(windowTextEntry);
			textscroll.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);

			textWindow.setLayout(new BorderLayout());
			
			textWindow.getContentPane().add(textscroll,BorderLayout.CENTER);
			JButton enter=new JButton("Enter");
			enter.addActionListener(new WindowButtonListener());
			textWindow.getContentPane().add(enter,BorderLayout.SOUTH);
		}
		
		windowTextEntry.setText(textentry.getText());
		textWindow.setVisible(true);
	}

	private class WindowButtonListener implements ActionListener{
		public void actionPerformed(ActionEvent arg0) {
			String str=windowTextEntry.getText();
			textWindow.setVisible(false);
			textentry.setText(str);
			textentry.fireAction();
		}
	}
	
	private class FireableTextField extends JTextField implements KeyListener,ActionListener{
		private LinkedList<String> values=new LinkedList<String>();
		private int hist=-1;
		private String working="";
		public FireableTextField(){
			addKeyListener(this);
			addActionListener(this);
		}
		public void fireAction(){
			fireActionPerformed();
		}

		public void keyReleased(KeyEvent event) {}
		public void keyTyped(KeyEvent event) {}
		public void keyPressed(KeyEvent event) {
			if(event.getKeyCode()==KeyEvent.VK_UP){
				if(hist==-1){
					working=getText();
				}else{
					values.set(hist, getText());
				}
				if(hist<values.size()-1){
					hist++;
					setText(values.get(hist));
				}
			}else if(event.getKeyCode()==KeyEvent.VK_DOWN){
				if(hist==-1){
					working=getText();
				}
				else{
					values.set(hist, getText());
				}
				
				if(hist>0){
					hist--;
					setText(values.get(hist));
				}else if(hist>-1){
					hist--;
					setText(working);
				}
			}
		}
		public void actionPerformed(ActionEvent evt) {
			String str=getText();
			values.addFirst(str);
			hist=-1;
			working="";
			setText(working);
			for(MapPanelListener m : listeners){
				m.textEntered(str);
			}
		}
	}

	public JTextField getTextEntry() {
		return textentry;
	}
}